package thrust;

import java.awt.event.*;
import rtflow.RTflowCode;

/**
 * Class that handles user input and timer ticks. It merely implements
 * the two interfaces KeyListener and ActionListener to receive key
 * events and timer events. On a key event, the corresponding input
 * variable in the RTflowCode object is set or reset. On a timer event,
 * an execution cycle of the RTflowCode object is run, and the GameView
 * is repainted.
 */
public class GameController implements KeyListener, ActionListener {
	private GameView view;
	private RTflowCode rtflowCode;
	private int left, right, thrust, fire;
	private boolean[] boolArray;
	private int[] intArray;
	private float[] realArray;
	
	/**
	 * Constructor.
	 * @param view The component to repaint on each timer event. 
	 * @param rtflowCode The object to set input to and to execute.
	 */
	public GameController(GameView view, RTflowCode rtflowCode) {
		this.view = view;
		this.rtflowCode = rtflowCode;
		boolArray = rtflowCode.getBoolArray();
		intArray = rtflowCode.getIntArray();
		realArray = rtflowCode.getRealArray();

		// Find indexes of input variables in generated code
		left = RTflowCode.findName(rtflowCode.boolNames, "left");
		right = RTflowCode.findName(rtflowCode.boolNames, "right");
		thrust = RTflowCode.findName(rtflowCode.boolNames, "thrust");
		fire = RTflowCode.findName(rtflowCode.boolNames, "fire");
	}
	
	/**
	 * Handles a timer event.
	 */
	public void actionPerformed(ActionEvent e) {
		rtflowCode.run();
		view.repaint();
	}

	/**
	 * Handle a key down event.
	 */
	public void keyPressed(KeyEvent e) {
		// If it's any of the game control keys, set the corresponding input
		// variable in the RTflowCode object.
		switch (e.getKeyCode()) {
		case KeyEvent.VK_LEFT:
			boolArray[left] = true;
			break;
		case KeyEvent.VK_RIGHT:
			boolArray[right] = true;
			break;
		case KeyEvent.VK_DOWN:
			boolArray[thrust] = true;
			break;
		case KeyEvent.VK_SPACE:
			boolArray[fire] = true;
			break;
		}
	}

	/**
	 * Handles a key up event.
	 */
	public void keyReleased(KeyEvent e) {
		// If it's any of the game control keys, reset the corresponding input
		// variable in the RTflowCode object.
		switch (e.getKeyCode()) {
		case KeyEvent.VK_LEFT:
			boolArray[left] = false;
			break;
		case KeyEvent.VK_RIGHT:
			boolArray[right] = false;
			break;
		case KeyEvent.VK_DOWN:
			boolArray[thrust] = false;
			break;
		case KeyEvent.VK_SPACE:
			boolArray[fire] = false;
			break;
		}
	}

	public void keyTyped(KeyEvent e) {
	}
}
